--===========================================================================--
--
-- Rxunit2 Test Bench
--
--
-- John Kent 16th February 2003
--
--
-------------------------------------------------------------------------------
library ieee;
   use ieee.std_logic_1164.all;
   use IEEE.STD_LOGIC_ARITH.ALL;
   use IEEE.STD_LOGIC_UNSIGNED.ALL;
   use ieee.numeric_std.all;

entity rxunit2_testbench is
end rxunit2_testbench;

-------------------------------------------------------------------------------
-- Architecture for memio Controller Unit
-------------------------------------------------------------------------------
architecture behavior of rxunit2_testbench is
  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------
  -- CPU Interface signals
  signal SysClk       : Std_Logic;
  signal uart_reset   : Std_Logic;
  signal uart_data_out: Std_Logic_Vector(7 downto 0);
  signal rxbit        : Std_Logic;
  signal EnabRX       : Std_Logic;
  signal Read         : Std_Logic;
  signal CtrlReg      : std_logic_vector(2 downto 0);
  signal FErr         : std_logic;
  signal OErr         : std_logic;
  signal PErr         : std_logic;
  signal DRdy         : std_logic;


-----------------------------------------------------------------
--
-- Open Cores Mini UART
--
-----------------------------------------------------------------
  -----------------------------------------------------------------------------
  -- Receive Unit
  -----------------------------------------------------------------------------
  component RxUnit
  port (
     Clk    : in  Std_Logic;  -- Clock signal
     Reset  : in  Std_Logic;  -- Reset input
     Enable : in  Std_Logic;  -- Enable input
     RxD    : in  Std_Logic;  -- RS-232 data input
     ReadD  : in  Std_Logic;  -- Read data signal
     Format : in  Std_Logic_Vector(2 downto 0); -- word format
     FRErr  : out Std_Logic;  -- Status signal
     ORErr  : out Std_Logic;  -- Status signal
	  PAErr  : out Std_logic;  -- Status signal
     DARdy  : out Std_Logic;  -- Status signal
     DAOut  : out Std_Logic_Vector(7 downto 0));
  end component;

begin

  -----------------------------------------------------------------------------
  -- Instantiation of internal components
  -----------------------------------------------------------------------------


  RxDev   : RxUnit  port map (
                Clk      => SysClk,
					 Reset    => uart_reset,
					 Enable   => EnabRX,
					 RxD      => rxbit,
					 ReadD    => Read,
					 Format   => CtrlReg(2 downto 0),
					 FRErr    => FErr,
					 ORErr    => OErr,
					 PAErr    => PErr,
					 DARdy    => DRdy,
					 DAOut    => uart_data_out );


  -- *** Test Bench - User Defined Section ***
   tb : PROCESS
	variable count : integer;
   BEGIN

   CtrlReg <= "101";
		for count in 0 to 4096 loop
			case count is
			when 0 =>
				uart_reset <= '1';
				rxbit <= '1';
				Read <= '0';
			when 1 =>
				uart_reset <= '0';
			when 256 =>
            rxbit <= '0'; -- start
			when 512 =>
			   rxbit <= '1'; -- bit 0
			when 768 =>
            rxbit <= '0'; -- bit 1
			when 1024 =>
			   rxbit <= '1'; -- bit 2
			when 1280 =>
            rxbit <= '1'; -- bit3
			when 1536 =>
			   rxbit <= '0'; -- bit 4
			when 1792 =>
            rxbit <= '0'; -- bit 5
			when 2048 =>
			   rxbit <= '1'; -- bit 6
			when 2304 =>
            rxbit <= '0'; -- bit 7
			when 2560 =>
			   rxbit <= '1'; -- stop 1
			when 2816 =>
			   rxbit <= '1'; -- stop 2
			when 3100 =>
				Read <= '1';
			when 3101 =>
				Read <= '0';
			when others =>
			   null;
			end case;
			if (count mod 16) = 0 then
            EnabRx <= '1';
			else
			   EnabRx <= '0';
			end if;
			SysClk <= '1';
			wait for 100 ns;
			SysClk <= '0';
			wait for 100 ns;
		end loop;

      wait; -- will wait forever
   END PROCESS;
-- *** End Test Bench - User Defined Section ***

end behavior; --===================== End of architecture =======================--

