--===========================================================================--
--
-- Txunit2 Test Bench
--
--
-- John Kent 16th February 2003
--
--
-------------------------------------------------------------------------------
library ieee;
   use ieee.std_logic_1164.all;
   use IEEE.STD_LOGIC_ARITH.ALL;
   use IEEE.STD_LOGIC_UNSIGNED.ALL;
   use ieee.numeric_std.all;

entity txunit2_testbench is
end txunit2_testbench;

-------------------------------------------------------------------------------
-- Architecture for memio Controller Unit
-------------------------------------------------------------------------------
architecture behavior of txunit2_testbench is
  -----------------------------------------------------------------------------
  -- Signals
  -----------------------------------------------------------------------------
  -- CPU Interface signals
  signal SysClk       : Std_Logic;
  signal Reset        : Std_Logic;
  signal TxData       : Std_Logic_Vector(7 downto 0);
  signal txbit        : Std_Logic;
  signal EnabTx       : Std_Logic;
  signal Load         : Std_Logic;
  signal CtrlReg      : std_logic_vector(2 downto 0);
  signal TBufE        : std_logic;


-----------------------------------------------------------------
--
-- Open Cores Mini UART
--
-----------------------------------------------------------------
  -----------------------------------------------------------------------------
  -- Transmitter Unit
  -----------------------------------------------------------------------------
  component TxUnit
  port (
     Clk    : in  Std_Logic;  -- Clock signal
     Reset  : in  Std_Logic;  -- Reset input
     Enable : in  Std_Logic;  -- Enable input
     LoadD  : in  Std_Logic;  -- Load transmit data
     Format : in  Std_Logic_Vector(2 downto 0); -- word format
     TxD    : out Std_Logic;  -- RS-232 data output
     TBE    : out Std_Logic;  -- Tx buffer empty
     DataO  : in  Std_Logic_Vector(7 downto 0));
  end component;

begin

  -----------------------------------------------------------------------------
  -- Instantiation of internal components
  -----------------------------------------------------------------------------

 
  TxDev   : TxUnit  port map (
                Clk      => SysClk,
					 Reset    => Reset,
					 Enable   => EnabTX,
					 LoadD    => Load,
					 Format   => CtrlReg(2 downto 0),
					 TxD      => txbit,
					 TBE      => TBufE,
					 DataO    => TxData);



  -- *** Test Bench - User Defined Section ***
   tb : PROCESS
	variable count : integer;
   BEGIN

      CtrlReg <= "101";
		for count in 0 to 4096 loop
			case count is
			when 0 =>
				reset <= '1';
				Load <= '0';
				TxData <= "00000000";
			when 1 =>
				reset <= '0';
			when 2 =>
		      TxData <= "01010011";
				Load   <= '1';
			when 3 =>
				Load   <= '0';
			when others =>
			   null;
			end case;
			if (count mod 256) = 0 then
            EnabTx <= '1';
			else
			   EnabTx <= '0';
			end if;
			SysClk <= '1';
			wait for 100 ns;
			SysClk <= '0';
			wait for 100 ns;
		end loop;

      wait; -- will wait forever
   END PROCESS;
-- *** End Test Bench - User Defined Section ***

end behavior; --===================== End of architecture =======================--

